@@ -16,7 +16,7 @@ from examples.connect import EXAMPLE_IMAGE_NAME
 Create resources with the Image service.
 
 For a full guide see
-http://developer.openstack.org/sdks/python/openstacksdk/user/guides/image.html
+https://docs.openstack.org/openstacksdk/latest/user/guides/image.html
 """
 
 
@@ -24,8 +24,10 @@ def import_image(conn):
     print("Import Image:")
 
     # Url where glance can download the image
-    uri = 'https://download.cirros-cloud.net/0.4.0/' \
-          'cirros-0.4.0-x86_64-disk.img'
+    uri = (
+        'https://download.cirros-cloud.net/0.4.0/'
+        'cirros-0.4.0-x86_64-disk.img'
+    )
 
     # Build the image attributes and import the image.
     image_attrs = {
