@@ -10,11 +10,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-# import types so that we can reference ListType in sphinx param declarations.
-# We can't just use list, because sphinx gets confused by
-# openstack.resource.Resource.list and openstack.resource2.Resource.list
-import types  # noqa
-
 from openstack.cloud import _utils
 from openstack.cloud import exc
 from openstack.image.v2._proxy import Proxy
@@ -35,21 +30,6 @@ class ImageCloudMixin:
     def __init__(self):
         self.image_api_use_tasks = self.config.config['image_api_use_tasks']
 
-    @property
-    def _raw_image_client(self):
-        if 'raw-image' not in self._raw_clients:
-            image_client = self._get_raw_client('image')
-            self._raw_clients['raw-image'] = image_client
-        return self._raw_clients['raw-image']
-
-    @property
-    def _image_client(self):
-        if 'image' not in self._raw_clients:
-            self._raw_clients['image'] = self._get_versioned_client(
-                'image', min_version=1, max_version='2.latest'
-            )
-        return self._raw_clients['image']
-
     def search_images(self, name_or_id=None, filters=None):
         images = self.list_images()
         return _utils._filter_list(images, name_or_id, filters)
@@ -71,7 +51,7 @@ class ImageCloudMixin:
         images = []
         params = {}
         image_list = []
-        if self._is_client_version('image', 2):
+        if utils.supports_version(self.image, '2'):
             if show_all:
                 params['member_status'] = 'all'
         image_list = list(self.image.images(**params))
