@@ -17,7 +17,6 @@ import fnmatch
 import functools
 import inspect
 import re
-import time
 import uuid
 
 from decorator import decorator
@@ -31,24 +30,6 @@ from openstack.cloud import exc
 _decorated_methods = []
 
 
-def _make_unicode(input):
-    """Turn an input into unicode unconditionally
-
-    :param input: A unicode, string or other object
-    """
-    try:
-        if isinstance(input, unicode):
-            return input
-        if isinstance(input, str):
-            return input.decode('utf-8')
-        else:
-            # int, for example
-            return unicode(input)
-    except NameError:
-        # python3!
-        return str(input)
-
-
 def _dictify_resource(resource):
     if isinstance(resource, list):
         return [_dictify_resource(r) for r in resource]
@@ -80,6 +61,7 @@ def _filter_list(data, name_or_id, filters):
         OR
 
         A string containing a jmespath expression for further filtering.
+        Invalid filters will be ignored.
     """
     # The logger is openstack.cloud.fmmatch to allow a user/operator to
     # configure logging not to communicate about fnmatch misses
@@ -87,7 +69,7 @@ def _filter_list(data, name_or_id, filters):
     log = _log.setup_logging('openstack.fnmatch')
     if name_or_id:
         # name_or_id might already be unicode
-        name_or_id = _make_unicode(name_or_id)
+        name_or_id = str(name_or_id)
         identifier_matches = []
         bad_pattern = False
         try:
@@ -99,8 +81,8 @@ def _filter_list(data, name_or_id, filters):
             # search
             fn_reg = None
         for e in data:
-            e_id = _make_unicode(e.get('id', None))
-            e_name = _make_unicode(e.get('name', None))
+            e_id = str(e.get('id', None))
+            e_name = str(e.get('name', None))
 
             if (e_id and e_id == name_or_id) or (
                 e_name and e_name == name_or_id
@@ -131,6 +113,17 @@ def _filter_list(data, name_or_id, filters):
         if not d:
             return False
         for key in f.keys():
+            if key not in d:
+                log.warning(
+                    "Invalid filter: %s is not an attribute of %s.%s",
+                    key,
+                    e.__class__.__module__,
+                    e.__class__.__qualname__,
+                )
+                # we intentionally skip this since the user was trying to
+                # filter on _something_, but we don't know what that
+                # _something_ was
+                raise AttributeError(key)
             if isinstance(f[key], dict):
                 if not _dict_filter(f[key], d.get(key, None)):
                     return False
@@ -140,15 +133,8 @@ def _filter_list(data, name_or_id, filters):
 
     filtered = []
     for e in data:
-        filtered.append(e)
-        for key in filters.keys():
-            if isinstance(filters[key], dict):
-                if not _dict_filter(filters[key], e.get(key, None)):
-                    filtered.pop()
-                    break
-            elif e.get(key, None) != filters[key]:
-                filtered.pop()
-                break
+        if _dict_filter(filters, e):
+            filtered.append(e)
     return filtered
 
 
@@ -374,58 +360,6 @@ def safe_dict_max(key, data):
     return max_value
 
 
-def _call_client_and_retry(
-    client, url, retry_on=None, call_retries=3, retry_wait=2, **kwargs
-):
-    """Method to provide retry operations.
-
-    Some APIs utilize HTTP errors on certain operations to indicate that
-    the resource is presently locked, and as such this mechanism provides
-    the ability to retry upon known error codes.
-
-    :param object client: The client method, such as:
-        ``self.baremetal_client.post``
-    :param string url: The URL to perform the operation upon.
-    :param integer retry_on: A list of error codes that can be retried on.
-        The method also supports a single integer to be
-        defined.
-    :param integer call_retries: The number of times to retry the call upon
-        the error code defined by the 'retry_on' parameter. Default: 3
-    :param integer retry_wait: The time in seconds to wait between retry
-        attempts. Default: 2
-
-    :returns: The object returned by the client call.
-    """
-
-    # NOTE(TheJulia): This method, as of this note, does not have direct
-    # unit tests, although is fairly well tested by the tests checking
-    # retry logic in test_baremetal_node.py.
-    log = _log.setup_logging('shade.http')
-
-    if isinstance(retry_on, int):
-        retry_on = [retry_on]
-
-    count = 0
-    while count < call_retries:
-        count += 1
-        try:
-            ret_val = client(url, **kwargs)
-        except exc.OpenStackCloudHTTPError as e:
-            if retry_on is not None and e.response.status_code in retry_on:
-                log.debug(
-                    'Received retryable error %(err)s, waiting '
-                    '%(wait)s seconds to retry',
-                    {'err': e.response.status_code, 'wait': retry_wait},
-                )
-                time.sleep(retry_wait)
-                continue
-            else:
-                raise
-        # Break out of the loop, since the loop should only continue
-        # when we encounter a known connection error.
-        return ret_val
-
-
 def parse_range(value):
     """Parse a numerical range string.
 
