@@ -18,6 +18,7 @@ from openstack import exceptions
 from openstack.image.v1 import image as _image
 from openstack import proxy
 from openstack import utils
+from openstack import warnings as os_warnings
 
 
 def _get_name_and_filename(name, image_format):
@@ -258,7 +259,10 @@ class Proxy(proxy.Proxy):
         :returns: The results of image creation
         :rtype: :class:`~openstack.image.v1.image.Image`
         """
-        warnings.warn("upload_image is deprecated. Use create_image instead.")
+        warnings.warn(
+            "upload_image is deprecated. Use create_image instead.",
+            os_warnings.OpenStackDeprecationWarning,
+        )
         return self._create(_image.Image, **attrs)
 
     def _upload_image(
@@ -386,7 +390,7 @@ class Proxy(proxy.Proxy):
         image,
         stream=False,
         output=None,
-        chunk_size=1024,
+        chunk_size=1024 * 1024,
     ):
         """Download an image
 
@@ -411,7 +415,7 @@ class Proxy(proxy.Proxy):
             When ``False``, return the entire contents of the response.
         :param output: Either a file object or a path to store data into.
         :param int chunk_size: size in bytes to read from the wire and buffer
-            at one time. Defaults to 1024
+            at one time. Defaults to 1024 * 1024 = 1 MiB
 
         :returns: When output is not given - the bytes comprising the given
             Image when stream is False, otherwise a :class:`requests.Response`
