@@ -9,7 +9,8 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
-from typing import List
+
+import typing as ty
 
 from openstack.common import tag
 from openstack.network.v2 import _base
@@ -58,7 +59,7 @@ class Port(_base.NetworkResource, tag.TagMixin):
     # Properties
     #: Allowed address pairs list. Dictionary key ``ip_address`` is required
     #: and key ``mac_address`` is optional.
-    allowed_address_pairs: List[dict] = resource.Body(
+    allowed_address_pairs: ty.List[dict] = resource.Body(
         'allowed_address_pairs', type=list
     )
     #: The ID of the host where the port is allocated. In some cases,
